/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *       - test it
  */
package jaugment2.AR.marker.artoolkit;

/**
 * Title:        ARUI
 * Description:  native class wrapping ARTookit for marker-detection
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */
public class JNIARToolkitImpl
{

 static
 {
  System.loadLibrary("artoolkitjni");
 }


 /**
  * initialize ARToolkit
  * @param xres = x-resolution of the frames
  * @param yres = y-resolution of the frames
  */
 private native static void init(int xres, int yres);

 /**
  * clean-up any native resources
  */
 private native static void deinit();

 private static final String imageformat = "RGB"; // BGR, ABGR

 /**
  * Find markers in the given image.
  */
 protected native static void processImage(byte b[]);


 private static boolean initialized = false;

  public JNIARToolkitImpl(int xres, int yres) throws Exception
  {
   synchronized(imageformat)
    {
     if(initialized)
       throw new IllegalStateException("only one instance of JNIARToolkitImpl can be initialized at once");
     init(xres, yres);
     initialized = true;
    }
  }


  public void dispose() throws Exception
  {
   synchronized(imageformat)
    {
     if(initialized)
       return;
     deinit();
     initialized = false;
    }
  }


  public void processFrame(byte buffer[]) throws Exception
  {
   processImage(buffer);
  }



}