/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-08, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-02-22, Marcus Wolschon<Suran@gmx.net>)
  *       - content is invisible at startup
  * 0.0.3 (2002-02-22, Marcus Wolschon<Suran@gmx.net>)
  *       - clicking on icons works fully
  * 0.0.4 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *       - supports ARUIWidgetFactory
  *       - automatically shows new representations
  * 0.0.5 (2002-03-02, Marcus Wolschon<Suran@gmx.net>)
  *       - uses a Panel as a top-level widget
  *       - testcode for serialization
  * 0.0.6 (2002-03-03, Marcus Wolschon<Suran@gmx.net>)
  *       - loads layout from config-file
  * -------------------------------------------------------------------
  * TODO:
  */

package jaugment2.UI.ARUI;

   // JDK
import java.awt.*;
import java.awt.event.*;
   // Swing
import javax.swing.*;
   // JINI
import net.jini.core.lookup.ServiceTemplate;
   // project
import jaugment2.helpers.Logger;
import jaugment2.UI.UIManager.*;
import jaugment2.UI.ARUI.widgets.*;

/**
 * <p>Title: ARUI</p>
 * <p>Description: Augmented-Reality UI</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */
public class ARUIMain extends JPanel implements jaugment2.UI.UIManager.RepresentationSeekerEventListener
{

  private UIManagerBean uiManagerBean;
  private static final boolean debug = true;

  public static void main(String[] args)
  {

   JFrame f = new JFrame("jAugment experimental Augmented-Reality UI");
   f.getContentPane().setLayout(new BorderLayout());
   f.getContentPane().add(new ARUIMain(), BorderLayout.CENTER);
   f.setSize(new Dimension(640, 480));
   f.show();

  }


  private jaugment2.UI.ARUI.widgets.Panel content = new jaugment2.UI.ARUI.widgets.Panel();

  public ARUIMain()
  {
   try
     {
        // enable the window-close event
      enableEvents(AWTEvent.WINDOW_EVENT_MASK);

     // open serial-port
     //TODO: This is a workaround to open ANY serial-port later, it seems the loadLibrary of the comm-API does not work after our UIManagerBean has been started.
     //if you do not use the SerialTextInputDevice you can comment-out this line
     try{try{javax.comm.CommPortIdentifier.getPortIdentifiers();}catch(Exception e){e.printStackTrace(Logger.err);}}catch(Error e){e.printStackTrace(Logger.err);}


        // TODO: final Layout will be described by an XML-structure defining the tree of Widgets and their properties and have Listeners atached to them for scripting

        try
          {
           content.setBackground(java.awt.Color.black);
           javax.xml.parsers.SAXParser parser = javax.xml.parsers.SAXParserFactory.newInstance().newSAXParser();

           org.xml.sax.Parser parser2 = parser.getParser();
//           jaugment2.UI.ARUI.xml.ChainedDocumentHandler dh = content.internalizeFromXML();
           jaugment2.UI.ARUI.xml.ChainedDocumentHandler dh = ARUIWidgetHelper.internalizeUI(content);
           dh.setParser(parser2, null);
           parser2.setDocumentHandler(dh);

           parser2.parse( new org.xml.sax.InputSource(new java.io.FileInputStream(System.getProperty("jaugment2dir")+System.getProperty("file.separator")+"etc"+System.getProperty("file.separator")+"arui_layout.xml")) );
          }
        catch(Exception e)
          {
           e.printStackTrace(Logger.err);

           content = new jaugment2.UI.ARUI.widgets.Panel();

             // cannot load layout, building compiled-in default-layout
           content.setLayout(new BorderLayout());
           content.setBackground(java.awt.Color.black);

           ARWindow contentw = new ARWindow();
           ARUITopPanel top = new ARUITopPanel();

           content.add(top, BorderLayout.NORTH);
           content.add(contentw, BorderLayout.CENTER);

           contentw.setVisible(false);
          }


     setLayout(new BorderLayout());
     setBackground(Color.black);
     this.add(content, BorderLayout.CENTER);

       // register the top-level widgets with ARUIWidgetHelper
     ARUIWidgetHelper.addTopLevelWidget(content);






        // the RepresentationSeeker will do all the nasty low-level-work of the ServiceItems for us

     RepresentationSeeker seeker = new RepresentationSeeker();
                          seeker.setListener(this);
        // register the types of representations, that we can handle
                          seeker.addRepresentationFactoryType(jaugment2.UI.ARUI.jini.ARUIWidgetFactory.class, jaugment2.UI.ARUI.jini.ARUIWidgetFactory.class.getMethod("createARUIWidget", new Class[]{Object.class}), new Object[]{null});
                          seeker.addRepresentationFactoryType(net.jini.lookup.ui.factory.JComponentFactory.class, net.jini.lookup.ui.factory.JComponentFactory.class.getMethod("getJComponent", new Class[]{Object.class}), new Object[]{null});
                          seeker.addRepresentationFactoryType(net.jini.lookup.ui.factory.JComponentFactory.class, net.jini.lookup.ui.factory.PanelFactory.class.getMethod("getPanel", new Class[]{Object.class}), new Object[]{null});
                          seeker.addRepresentationType(java.awt.Component.class);

           // every UI uses this bean to keep track of everything and start-up apps
      this.uiManagerBean = new UIManagerBean(new ServiceTemplate(null, null, null), // look for every service out there
                                             seeker
                                            );

       // the starter-panel is needed
     this.RepresentationsFound(new Object[]{new jaugment2.UI.swingUI.StarterPanel(this.uiManagerBean, this.getFont())}, null/*TODOjava.awt.Image icon*/, "start"/*TODO: l10n*/, null);



     // ---debug-code
/*     try
       {
        java.io.StringWriter sw = new java.io.StringWriter();
        java.io.PrintWriter pw = new java.io.PrintWriter(sw);

        ARUIWidgetHelper.externalizeUI(content, pw);
//        content.externalizeToXML(pw);
        pw.close();
        String s = sw.toString();
        Logger.debug.println(s);


        javax.xml.parsers.SAXParser parser = javax.xml.parsers.SAXParserFactory.newInstance().newSAXParser();

        org.xml.sax.Parser parser2 = parser.getParser();
        jaugment2.UI.ARUI.xml.ChainedDocumentHandler dh = content.internalizeFromXML();
        dh.setParser(parser2, null);
        parser2.setDocumentHandler(dh);

        parser2.parse( new org.xml.sax.InputSource(new java.io.StringBufferInputStream(s)) );

       }
     catch(Throwable x)
       {
        x.printStackTrace(Logger.err);
       }*/

     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }



 public synchronized void RepresentationsFound(Object representations[], java.awt.Image icon, String localizedName, net.jini.core.lookup.ServiceID serviceid)
 {
  try
    {
     if(debug)Logger.debug.println("DEBUG[ARUIMain] RepresentationsFound() icon="+icon+" localizedName="+localizedName+" representations.length="+representations.length);



     IconSlot slot = content.getFreeIconSlot();
     if(slot == null)
       {
        Logger.err.println("ERROR[ARUIMain] RepresentationsFound() I have no free icon-slot for this application");
        return;
        }


     IconWidget widget = new IconWidget(icon, localizedName, representations.length==0?null:representations[0], serviceid);
     widget.addIconWidgetListener(iconclicklistener);
     slot.addIconWidget(widget);
     if(debug)Logger.debug.println("DEBUG[ARUIMain] RepresentationsFound() adding iconWidget "+widget.hashCode()+" to slot "+slot.hashCode());
        // show the new representation
     widget.toggleState();
    }
  catch(Exception e)
    {
     e.printStackTrace(Logger.err);
    }
 }
 public synchronized void RepresentationLost(Object representation, String localizedName, net.jini.core.lookup.ServiceID serviceid)
 {
  if(debug)Logger.debug.println("DEBUG[ARUIMain] RepresentationLost()");
 }

 /**
  * only thrown if a AdministrativePanel was requested
  * @param adminUI
  */
 public void AdministrativePanelLost(java.awt.Component adminUI)
 {
  if(debug)Logger.debug.println("DEBUG[ARUIMain] AdministrativePanelLost()");
 }





 //------------------------------------------- showing/hiding representations -------------------------------------------




 private IconWidget.IconWidgetListener iconclicklistener = new IconWidget.IconWidgetListener()
  {
   /**
    * returns true if the icon shall be placed into an ACTIVATED state
    */
   public boolean AppActivated(IconWidget widget, Object representation, net.jini.core.lookup.ServiceID serviceid, String title)
   {
    if(debug)Logger.debug.println("DEBUG[ARUIMain] AppActivated()");
    if(representation==null)
      {
       Logger.err.println("ERROR[ARUIMain] AppActivated() representation is null, I don't know what to do with it");
       return false;
      }


    if(representation instanceof java.awt.Component)
      {
       if(RepresentationSeeker.isDockin((java.awt.Component)representation  ))
         {
          Block blk = ARUIWidgetHelper.getFreeBlock(ARUIMain.this);
          if(blk == null)
            {  // we have no free blocks, use a Window
             if(debug)Logger.debug.println("DEBUG[ARUIMain] AppActivated() adding Dockin to a window, as we have no free blocks");
             ARWindow contentw = ARUIWidgetHelper.getFreeARWindow(content);
             if(contentw==null)
                      contentw = ARUIWidgetHelper.getAnyARWindow(content);
             if(contentw==null)
               {
                Logger.warning.println("WARNING[ARUIMain] no free Window found to show representation");
               }
             else
               {
                contentw.removeContent();
                contentw.setVisible(true);
                contentw.addContent(((java.awt.Component)representation), title);
               }
            }
          else
            {
             if(debug)Logger.debug.println("DEBUG[ARUIMain] AppActivated() adding Dockin to a Block");
             blk.addContent(((java.awt.Component)representation));
            }
         }
       else
         {
          if(debug)Logger.debug.println("DEBUG[ARUIMain] AppActivated() adding Representation(non-Dockin) to a Window");
          ARWindow contentw = ARUIWidgetHelper.getFreeARWindow(content);
          if(contentw==null)
                   contentw = ARUIWidgetHelper.getAnyARWindow(content);
          if(contentw==null)
            {
             Logger.warning.println("WARNING[ARUIMain] no free Window found to show representation");
            }
          else
            {
             contentw.removeContent();
             contentw.setVisible(true);
             contentw.addContent(((java.awt.Component)representation), title);
            }
         }
       return true;
      }
    else
      {
       Logger.err.println("ERROR[ARUIMain] AppActivated() representation is no component, I don't know what to do with it");
      }
    return false;
   }



   public void AppDeActivated(IconWidget widget, Object representation, net.jini.core.lookup.ServiceID serviceid) throws java.beans.PropertyVetoException
   {
//    if(debug)Logger.debug.println("DEBUG[ARUIMain] AppDeActivated() representation="+representation);
    if(representation==null)
      return;
    if(representation instanceof java.awt.Component)
      {
       Container parent = ((java.awt.Component)representation).getParent();
       if(parent==null)
         {
//          if(debug)Logger.debug.println("DEBUG[ARUIMain] AppDeActivated() parent==null");
          return;
         }
       parent.remove(((java.awt.Component)representation));

       ARWindow w = ARUIWidgetHelper.findARWindow(((java.awt.Component)representation));
       if(w==null)
         w = ARUIWidgetHelper.findARWindow2(((java.awt.Component)representation));
       if(w != null)
         {
          w.setVisible(false);
//          if(debug)Logger.debug.println("DEBUG[ARUIMain] AppDeActivated() found ARWindow");
         }

      }
   }
  };


}
