/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-20, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-03-01, Marcus Wolschon<Suran@gmx.net>)
  *       - added serialization/deserialization
  * 0.0.3 (2002-03-02, Marcus Wolschon<Suran@gmx.net>)
  *       - added getFreeARWindow  and getAnyARWindow
  * 0.0.4 (2002-03-03, Marcus Wolschon<Suran@gmx.net>)
  *       - made XML-output more pretty
  *       - fixed a formating-bug in XML-output
  *       - fixed XML-internalizing
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.UI.ARUI.widgets;

import javax.swing.JPanel;
import java.awt.*;
  // project
import jaugment2.helpers.Logger;
import jaugment2.UI.ARUI.xml.*;
  // SAX
import org.xml.sax.*;


/**
 * Title:        ARUI
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class ARUITopPanel extends JPanel implements ARUIWidgetIF
{
 private static final int southblockheight = 40;
 private static final int southblockwidth = 80;
 private static final int middleblockcount = 3;


 private static final int middleblockheight = 28;


 /**
  * width of the frame-polyline
  */
 private static final int framewidth = 5;
 /**
  * free space between border and widgets/widgets and widgets
  */
 private static final int spacebetweenwidgets = 2;

 /**
  * size of the edges (hor and vert)
  */
 private static final int frameedgesize = 15;

 private static final int southedgeheight = framewidth+3*spacebetweenwidgets+southblockheight+2*IconSlot.iconslotheight;
 private static final int southedgewidth = southblockwidth+2*spacebetweenwidgets;


 private IconSlot icons[] = new IconSlot[18];
 private Block blockleft = new Block();
 private Block blockright = new Block();

 private Block blockmiddle[] = new Block[middleblockcount];


 private void layoutchildren()
 {
  int iconheight = IconSlot.iconslotheight;
  int width = getWidth();
  int effectiveEdgeWidth = Math.max(frameedgesize, iconheight+framewidth/2+spacebetweenwidgets);
       // the first row of icons on both sides
   for(int i=0; i<4; i++)
     {
      icons[2*i].setBounds(framewidth/2+effectiveEdgeWidth+i*(iconheight+spacebetweenwidgets),
                         framewidth/2+spacebetweenwidgets,
                         iconheight,
                         iconheight);

      icons[2*i+1].setBounds(width-(iconheight+framewidth/2+effectiveEdgeWidth+i*(iconheight+spacebetweenwidgets)),
                         framewidth/2+spacebetweenwidgets,
                         iconheight,
                         iconheight);
     }

       // the second row of icons on both sides
   for(int i=0; i<5; i++)
     {
      icons[2*i+8].setBounds(framewidth/2+effectiveEdgeWidth+(i-1)*(iconheight+spacebetweenwidgets),
                         framewidth/2+2*spacebetweenwidgets+iconheight,
                         iconheight,
                         iconheight);

      icons[2*i+1+8].setBounds(width-(iconheight+framewidth/2+effectiveEdgeWidth+(i-1)*(iconheight+spacebetweenwidgets)),
                         framewidth/2+2*spacebetweenwidgets+iconheight,
                         iconheight,
                         iconheight);
     }
       // blocks below the icons
  blockleft.setBounds(spacebetweenwidgets+framewidth/2,
                      southedgeheight-(framewidth/2+southblockheight),
                      southblockwidth,
                      southblockheight);
  blockright.setBounds(width-(southblockwidth+spacebetweenwidgets+framewidth/2),
                       southedgeheight-(framewidth/2+southblockheight),
                       southblockwidth,
                       southblockheight);

     // blocks in the middle
  int middleblockstart = framewidth/2+effectiveEdgeWidth+4*(iconheight+spacebetweenwidgets);
  int middleblockend =   width-middleblockstart;
  int middleblockwidth = (((middleblockend-middleblockstart)-((blockmiddle.length-1)*spacebetweenwidgets))/blockmiddle.length);
  for(int i=0; i<blockmiddle.length; i++)
     {

      blockmiddle[i].setBounds(middleblockstart+(i*(middleblockwidth+spacebetweenwidgets)),
                              framewidth/2+spacebetweenwidgets,
                              middleblockwidth,
                              middleblockheight);
//      System.err.println("i="+i+"bound="+blockmiddle[i].getBounds());
     }
 }


  public ARUITopPanel()
  {
   setMinimumSize(new Dimension(320, southedgeheight+framewidth));
   setPreferredSize(new Dimension(Integer.MAX_VALUE, southedgeheight+framewidth));
   setBackground(Color.black);
   setLayout(null);
   setOpaque(false);



   internalizeFromXML();


   layoutchildren();
  }



  public void paint(Graphics g)
  {
   layoutchildren();
   try
     {
      int width = getWidth()-framewidth/2;
      int height = getHeight()-framewidth/2;
//      g.setColor(getBackground());
//      g.fillRect(0,0,getWidth(), getHeight());
      java.awt.Point p[] = new java.awt.Point[]
         {
          new java.awt.Point(frameedgesize, framewidth/2),
          new java.awt.Point(width-frameedgesize, framewidth/2),
          new java.awt.Point(width, frameedgesize),
          new java.awt.Point(width, southedgeheight),
          new java.awt.Point(width-southedgewidth, southedgeheight),
          new java.awt.Point(width-southedgewidth-frameedgesize, southedgeheight-frameedgesize),
          new java.awt.Point(southedgewidth+frameedgesize, southedgeheight-frameedgesize),
          new java.awt.Point(southedgewidth, southedgeheight),
          new java.awt.Point(framewidth/2, southedgeheight),
          new java.awt.Point(framewidth/2, frameedgesize),
          new java.awt.Point(frameedgesize, framewidth/2)
         };
      GraphicHelper.fillSmoothPolyline(p, framewidth, GraphicHelper.getOutherBorderColor(), GraphicHelper.getFillColor(), g);
      paintChildren(g);
      /*for(int i=0; i<icons.length; i++)
        icons[i].paint(g);*/
     }
   catch(Exception e)
     {
      e.printStackTrace();
     }
  };








 // ---------------------- ARUIWidgetIF ------------------------


 public IconSlot getFreeIconSlot()
 {
  return ARUIWidgetHelper.getFreeIconSlot(this);
 }
 public Block getFreeBlock()
 {
  return ARUIWidgetHelper.getFreeBlock(this);
 }

 public ARWindow getFreeARWindow()
 {
  return ARUIWidgetHelper.getFreeARWindow(this);
 }
 public ARWindow getAnyARWindow()
 {
  return ARUIWidgetHelper.getFreeARWindow(this);
 }

 //--------- externaline and internalize to/from XML------------


 private class MyChainedDocumentHandler extends ChainedDocumentHandler
 {
  public void endElement(String p0) throws SAXException
  {
   super.endElement(p0);
  }

  public void startElement(String p0, AttributeList p1) throws SAXException
  {
   if(getLevel()==0)
     {
      if(!p0.equalsIgnoreCase("ARUITopPanel"))
        throw new SAXException("expected: <ARUITopPanel>, found: <"+p0+">");

     }
   else
   if(getLevel()==1)  // ignore deeper levels, that may exist dues to serialization of content that we do not know of yet
     {

        // ignore other children, that may be added in later versions of this format
      if(p0.equalsIgnoreCase("icon"))
        {


         String posstr = p1.getValue("pos");
         try
           {
            IconSlot islt = icons[Integer.parseInt(posstr)];
            if(islt instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)islt).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }

            add(islt);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add icon at pos \""+posstr+"\", "+e.getMessage());
           }
        }


      if(p0.equalsIgnoreCase("blockleft"))
        {
         try
           {
            Block blk = blockleft;
            if(blk instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)blk).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }

            add(blk);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add left block, "+e.getMessage());
           }
        }

      if(p0.equalsIgnoreCase("blockright"))
        {
         try
           {
            Block blk = blockright;
            if(blk instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)blk).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }

            add(blk);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add left block, "+e.getMessage());
           }
        }

      if(p0.equalsIgnoreCase("blockmiddle"))
        {

         String posstr = p1.getValue("pos");
         try
           {
            Block blk = blockmiddle[Integer.parseInt(posstr)];
            if(blk instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)blk).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }

            add(blk);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add block at pos "+posstr+", "+e.getMessage());
           }
        }



     }
   super.startElement(p0, p1);
  }
 }

 public void externalizeToXML(java.io.PrintWriter pw)
 {
  pw.print("<ARUITopPanel>"); //TODO: constants should be variable

  for(int i=0; i<icons.length; i++)
    {
     pw.print("<icon pos=\""+i+"\">");

     if(icons[i] instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)icons[i]).externalizeToXML(pw);
       }
     pw.println("</icon>");
    };

  if(blockleft != null)
    {
     pw.print("<blockleft>");

     if(blockleft instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)blockleft).externalizeToXML(pw);
       }
     pw.println("</blockleft>");
    };

  for(int i=0; i<blockmiddle.length; i++)
    {
     pw.print("<blockmiddle pos=\""+i+"\">");

     if(blockmiddle[i] instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)blockmiddle[i]).externalizeToXML(pw);
       }
     pw.println("</blockmiddle>");
    };

  if(blockright != null)
    {
     pw.print("<blockright>");

     if(blockright instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)blockright).externalizeToXML(pw);
       }
     pw.println("</blockright>");
    };



  pw.println("</ARUITopPanel>");
 }

 public ChainedDocumentHandler internalizeFromXML()
 {
     // clean up
  removeAll();
  if(icons != null)
   for(int i=0; i<icons.length; i++)
     {
      if(icons[i] != null)
      remove(icons[i]);
     }
  if(blockmiddle != null)
   for(int i=0; i<blockmiddle.length; i++)
     {
      if(blockmiddle[i] != null)
      remove(blockmiddle[i]);
     }
  if(blockleft != null)
     remove(blockleft);
  if(blockright != null)
     remove(blockright);


     // rebuild
   for(int i=0; i<icons.length; i++)
     {
      icons[i] = new IconSlot();
      add(icons[i]);
     }
   for(int i=0; i<blockmiddle.length; i++)
     {
      blockmiddle[i] = new Block();
      add(blockmiddle[i]);
     }
   add(blockleft);
   add(blockright);

  return new MyChainedDocumentHandler();
 }
}
