/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-10, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-03-03, Marcus Wolschon<Suran@gmx.net>)
  *       - implemented first code for scripting
  * 0.0.3 (2002-03-04, Marcus Wolschon<Suran@gmx.net>)
  *       - implemented reading scripts (untested)
  * -------------------------------------------------------------------
  * TODO:
  *       - externalize/internalize scripts
  *       - allow registration and editing of scripts
  *       - test scripts
  *       - register ScriptActionLister and ScriptPropertyChangeListener with the object
  *       - write default-scripts
  */
package jaugment2.UI.ARUI.widgets;

   // JDK
import java.util.*;
import java.beans.*;
   // J2EE/SAX-parser
import org.xml.sax.*;
   // project
import jaugment2.helpers.Logger;
import jaugment2.UI.ARUI.xml.*;

/**
 * Title:        ARUI
 * Description:  class with helper-functions used to implement the functionality of ARUIWidgetIF
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class ARUIWidgetHelper
{
 private static final boolean debug = true;

 private static LinkedList toplevel = new LinkedList();
 public static void addTopLevelWidget(ARUIWidgetIF w)
 {
  toplevel.add(w);
 }



 /**
  * search all sub-components for the first free IconSlot found
  */
 public static IconSlot getFreeIconSlot(java.awt.Container This)
 {
  try
    {
     Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeIconSlot() We have "+This.getComponentCount()+" sub-components");
     for(int i=0; i<This.getComponentCount(); i++)
        if(This.getComponent(i) instanceof ARUIWidgetIF)
          {
           IconSlot retval = ((ARUIWidgetIF)This.getComponent(i)).getFreeIconSlot();
           if(retval != null)
             return retval;
          }

     return null;
    }
  catch(Exception e)
    {
     e.printStackTrace(jaugment2.helpers.Logger.err);
    }
  return null;
 };


 /**
  * search all sub-components for the first free IconSlot found
  */
 public static Block getFreeBlock(java.awt.Container This)
 {
  try
    {
//     Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeBlock() We have "+This.getComponentCount()+" sub-components");
     for(int i=0; i<This.getComponentCount(); i++)
        if(This.getComponent(i) instanceof ARUIWidgetIF)
          {
           Block retval = ((ARUIWidgetIF)This.getComponent(i)).getFreeBlock();
           if(retval != null)
             {
              Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeBlock() returning a block");
              return retval;
             }
          }
     Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeBlock() returning null");
     return null;
    }
  catch(Exception e)
    {
     e.printStackTrace(jaugment2.helpers.Logger.err);
     return null;
    }
 };


 /**
  * search all sub-components for the first free ARWindow found
  */
 public static ARWindow getFreeARWindow(java.awt.Container This)
 {
  try
    {
//     Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeBlock() We have "+This.getComponentCount()+" sub-components");
     for(int i=0; i<This.getComponentCount(); i++)
        if(This.getComponent(i) instanceof ARWindow)
          {
           ARWindow retval = ((ARUIWidgetIF)This.getComponent(i)).getFreeARWindow();
           if(retval != null)
             {
              Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeARWindow() returning a block");
              return retval;
             }
          }
     Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeARWindow() returning null");
     return null;
    }
  catch(Exception e)
    {
     e.printStackTrace(jaugment2.helpers.Logger.err);
     return null;
    }
 };

 /**
  * search all sub-components for the first free ARWindow found
  */
 public static ARWindow getAnyARWindow(java.awt.Container This)
 {
  try
    {
//     Logger.debug.println("DEBUG[ARUIWidgetHelper] getFreeBlock() We have "+This.getComponentCount()+" sub-components");
     for(int i=0; i<This.getComponentCount(); i++)
        if(This.getComponent(i) instanceof ARWindow)
          {
           ARWindow retval = ((ARUIWidgetIF)This.getComponent(i)).getAnyARWindow();
           if(retval != null)
             {
              Logger.debug.println("DEBUG[ARUIWidgetHelper] getAnyARWindow() returning a block");
              return retval;
             }
          }
     Logger.debug.println("DEBUG[ARUIWidgetHelper] getAnyARWindow() returning null");
     return null;
    }
  catch(Exception e)
    {
     e.printStackTrace(jaugment2.helpers.Logger.err);
     return null;
    }
 };




 public static ARWindow findARWindow2(java.awt.Component c)
 {
  Iterator iter = toplevel.iterator();
  while(iter.hasNext())
    {
     ARUIWidgetIF w = (ARUIWidgetIF)iter.next();
     if(w instanceof ARWindow)
       {
        ARWindow win = (ARWindow)w;
       if(win.getContent() == c)
         return win;
       }
    }
  return null;
 }

 /**
  * go up the tree of widgets to find an ARWindows
  */
 public static ARWindow findARWindow(java.awt.Component c)
 {
//  if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper] findARWindow(c"+(c==null?"=":"!")+"=null)");
  if(c == null)
    {
//     if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper] findARWindow(c==null)");
     return null;
    }
  if(c instanceof ARWindow)
    {
//     if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper] findARWindow(c instanceof ARWindow)");
     return (ARWindow)c;
    }
  java.awt.Container c2 = c.getParent();
  if(c2 == null)
    {
//     if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper] findARWindow(c.parent==null)");
     return null;
    }
  return findARWindow(c2);
 }




 //-------------------------------- scripts -----------------------------------


 private static HashMap class2defaultActionScript = new HashMap();
 private static HashMap instance2ActionScript = new HashMap();
 private static HashMap class2defaultPropertyChangeScript = new HashMap();
 private static HashMap instance2PropertyChangeScript = new HashMap();

 public void addDefaultActionListenerScript(Class c, String language, String script)
 {

 }




 private com.ibm.bsf.BSFManager mngr;
 /**
  * PropertyChangeListener, that invokes registred scripts
  */
 private java.beans.PropertyChangeListener ScriptPropertyChangeListener = new java.beans.PropertyChangeListener()
 {

    /**
     * This method gets called when a bound property is changed.
     * @param evt A PropertyChangeEvent object describing the event source
     *   	and the property that has changed.
     */

    public void propertyChange(PropertyChangeEvent evt)
    {
     try
       {
        Object source = evt.getSource();


        //TODO: invoke script for this instance
        Script script = (Script)instance2PropertyChangeScript.get(source);
        if(script != null)
          {
           if(mngr == null)
             mngr = new com.ibm.bsf.BSFManager();
           //TODO: set property that was changes as parameter/semi-global variable
           script.exec(mngr);
           return;
          }

        //TODO: invoke default-script for this class


               script = (Script)class2defaultPropertyChangeScript.get(source.getClass());
        if(script != null)
          {
           if(mngr == null)
             mngr = new com.ibm.bsf.BSFManager();
           //TODO: set property that was changes as parameter/semi-global variable
           script.exec(mngr);
           return;
          }

        //TODO: invoke global default-script (if any)


       }
     catch(Exception x)
       {
        x.printStackTrace(Logger.err);
       }
    }
 };

 /**
  * PropertyChangeListener, that invokes registred scripts
  */
 private java.awt.event.ActionListener ScriptActionListener = new java.awt.event.ActionListener()
 {

    /**
     * Invoked when an action occurs.
     */
    public void actionPerformed(java.awt.event.ActionEvent evt)
    {
     try
       {
        Object source = evt.getSource();


        //TODO: invoke script for this instance
        Script script = (Script)instance2ActionScript.get(source);
        if(script != null)
          {
           if(mngr == null)
             mngr = new com.ibm.bsf.BSFManager();
           //TODO: set property that was changes as parameter/semi-global variable
           script.exec(mngr);
           return;
          }

        //TODO: invoke default-script for this class


               script = (Script)class2defaultActionScript.get(source.getClass());
        if(script != null)
          {
           if(mngr == null)
             mngr = new com.ibm.bsf.BSFManager();
           //TODO: set property that was changes as parameter/semi-global variable
           script.exec(mngr);
           return;
          }

        //TODO: invoke global default-script (if any)


       }
     catch(Exception x)
       {
        x.printStackTrace(Logger.err);
       }
    }
 };



 private static class Script
 {
  private String lang;
  private String source = null;
  private int lineNo = 0;
  private int columnNo = 0;
  private Object script;


  public Script(Object script, String lang)
  {
   this.lang = lang;
   this.script = script;
  }

  public void exec(com.ibm.bsf.BSFManager mngr) throws com.ibm.bsf.BSFException
  {
   mngr.exec(lang, source, lineNo, columnNo, script);
  }


  public String getSourceCode()
  {
   if(script instanceof String)
     return script.toString();
   if(script instanceof StringBuffer)
     return script.toString();
   return null;
  }

  public void set(String sourcecode, String lang)
  {
   this.lang = lang;
   this.script = sourcecode;
  }


  public String getLanguage()
  {
   return lang;
  }

 }


 // ---------------------------------- XML ------------------------------------


 /**
  * externalize the whole UI with all elements, default- and instance-bound scripts
  */
 public static void externalizeUI(ARUIWidgetIF top, java.io.PrintWriter pw)
 {
  pw.println("<arui version=\"1.1\">");
  //TODO: externalize default scripts

  pw.println("<widget>");
  top.externalizeToXML(pw);
  pw.println("</widget>\n</arui>");
 }



 private static class MyChainedDocumentHandler extends ChainedDocumentHandler
 {
  private ARUIWidgetIF top;

  public MyChainedDocumentHandler(ARUIWidgetIF top)
  {
   this.top = top;
  }

  public void endElement(String p0) throws SAXException
  {
   super.endElement(p0);
  }
  private String currentscript_language = null;
  private String currentscript_type = null;
  private String currentscript_class = null;

  public void characters(char[] p0, int p1, int p2) throws SAXException
  {
   if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper]: MyChainedDocumentHandler.characters(\""+(new String(p0, p1, p2))+"\")");
     // only for scripts do we parse this
   if(currentscript_language != null)
     {
      String s = new String(p0, p1, p2);
      if(s.trim().length()==0)
        return;
        
      Script scr = new Script(s, currentscript_language);


      if(currentscript_type.equals("ActionListener"))
        {
         if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper]: adding script in language \""+currentscript_language+"\" to class2defaultActionScript");
         class2defaultActionScript.put(currentscript_class, scr);
        }
      if(currentscript_type.equals("PropertyChangeListener"))
        {
         if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper]: adding script in language \""+currentscript_language+"\" to class2defaultPropertyChangeScript");
         class2defaultPropertyChangeScript.put(currentscript_class, scr);
        }

     }
  }

  public void startElement(String p0, AttributeList p1) throws SAXException
  {
   if(getLevel()==0)
     {
      if(!p0.equalsIgnoreCase("arui"))
        throw new SAXException("expected: <arui>, found: <"+p0+">");

     }
   else
   if(getLevel()==1)  // ignore deeper levels, that may exist dues to serialization of content that we do not know of yet
     {

       //TODO: internalize default-scripts

        // ignore other children, that may be added in later versions of this format
      if(p0.equalsIgnoreCase("widget"))
        {
         ChainedDocumentHandler childhandler = top.internalizeFromXML();
         childhandler.setParser(getParser(), this);
         getParser().setDocumentHandler(childhandler);
        }
      else
      if(p0.equalsIgnoreCase("defaultscript"))
        {
         if(debug)Logger.debug.println("DEBUG[ARUIWidgetHelper]: MyChainedDocumentHandler.startElement(\""+p0+"\")");
         currentscript_language = p1.getValue("language");
         currentscript_class = p1.getValue("class");
         currentscript_type = p1.getValue("type");

         if(currentscript_language==null || currentscript_language.trim().length()<1)
            throw new SAXException("illegal defaultscript-tag, no attribute \"language\" ");

         if(currentscript_class==null || currentscript_class.trim().length()<1)
            throw new SAXException("illegal defaultscript-tag, no attribute \"class\" ");

         if(currentscript_type==null || currentscript_type.trim().length()<1)
            throw new SAXException("illegal defaultscript-tag, no attribute \"type\" ");

         if(!currentscript_type.equals("ActionListener") && !currentscript_type.equals("PropertyChangeListener"))
           {
            currentscript_language = null;
            Logger.warning.println("DEBUG[ARUIWidgetHelper]: unknown value for type-attribute of <defaultscript> \""+currentscript_type+"\", I know but \"ActionListener\" and \"PropertyChangeListener\", ignoring the script!");
           }
        }
      }
   super.startElement(p0, p1);
  }
 }



 public static jaugment2.UI.ARUI.xml.ChainedDocumentHandler internalizeUI(ARUIWidgetIF top)
 {
  // clear default-scripts
  class2defaultActionScript = new HashMap();
  instance2ActionScript = new HashMap();
  class2defaultPropertyChangeScript = new HashMap();
  instance2PropertyChangeScript = new HashMap();

  return new MyChainedDocumentHandler(top);
 }


}
