/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-21, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-02-22, Marcus Wolschon<Suran@gmx.net>)
  *       - debugged
  * 0.0.3 (2002-02-22, Marcus Wolschon<Suran@gmx.net>)
  *       - it finally works
  * 0.0.4 (2002-02-22, Marcus Wolschon<Suran@gmx.net>)
  *       - draws edges
  *       - fills edges
  * 0.0.5 (2002-03-01, Marcus Wolschon<Suran@gmx.net>)
  *       - added serialization/deserialization
  * 0.0.6 (2002-03-02, Marcus Wolschon<Suran@gmx.net>)
  *       - added getFreeARWindow  and getAnyARWindow
  *       - layouting children works
  * 0.0.7 (2002-03-03, Marcus Wolschon<Suran@gmx.net>)
  *       - tested internalizing
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.UI.ARUI.widgets;


import java.awt.*;
import javax.swing.*;
   // project
import jaugment2.helpers.Logger;
import jaugment2.UI.ARUI.xml.*;
  // SAX
import org.xml.sax.*;

/**
 * Title:        ARUI
 * Description:  widget, that displays a blue Window with the content of a fuly-opened Application
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class ARWindow extends javax.swing.JPanel implements ARUIWidgetIF
{
 private static final boolean debug = true;


 public IconSlot getFreeIconSlot()
 {
  return ARUIWidgetHelper.getFreeIconSlot(this);
 }
 public Block getFreeBlock()
 {
  return ARUIWidgetHelper.getFreeBlock(this);
 }

 public ARWindow getFreeARWindow()
 {
  return getContent()==null?this:null;
 }
 public ARWindow getAnyARWindow()
 {
  return this;
 }

 /**
  * width of the frame-polyline
  */
 private static final int framewidth = 5;
 /**
  * free space between border and widgets/widgets and widgets
  */
 private static final int spacebetweenwidgets = 2;

 /**
  * size of the edges (hor and vert) for the edges
  */
 private static final int frameedgesize = 25;//15;

 /**
  * size of the edges (hor and vert) for the outher border
  */
 private static final int outherframeedgesize = 15;

 /**
  * offset, that the outher frame is drawn in
  */
 private static final int outherframeoffset = 15;





 private Component contentp;
 private String title;

 private void layoutchildren()
 {
    // we cannot layout yet
  if(getWidth()<1)
    return;

  // manually layout this.content
  if(contentp != null)
    {
     contentp.setBounds(frameedgesize, frameedgesize, getWidth()-2*frameedgesize, getHeight()-2*frameedgesize);
     contentp.doLayout();
    if(debug)Logger.debug.println("DEBUG[ARWindow] layoutchildren() post content.bounds="+contentp.getBounds()+" content.parent="+contentp.getParent()+" isDisplayable="+isDisplayable()+" contentp.isDisplayable()="+contentp.isDisplayable()+" getWidth()="+getWidth());
    }

 }

  public ARWindow()
  {
    try
    {
      jbInit();
     setOpaque(false);
    }
    catch(Exception e)
    {
      e.printStackTrace(Logger.err);
    }
  }
  private void jbInit() throws Exception
  {
   this.setBackground(Color.black);
  }


  public void removeContent()
  {
   if(contentp != null)
      remove(contentp);
   contentp = null;
   setVisible(false);
  }

  public void addContent(Component c, String title)
  {
   if(contentp != null)
      remove(contentp);
   this.title = title;
   contentp = c;
   super.add(c);


   if(getWidth()>0)
     {
      layoutchildren();
      c.setSize(getWidth()-2*frameedgesize, getHeight()-2*frameedgesize);
      c.doLayout();
     }


   if(debug)Logger.debug.println("DEBUG[ARWindow] add("+c+") c.getParent="+c.getParent()+" content.ounds="+contentp.getBounds());
  }


  public void doLayout()
  {
   layoutchildren();
  }

  public void setBounds(int x, int y, int w, int h)
  {
   super.setBounds(x, y, w, h);
   layoutchildren();
  }


  public Component getContent()
  {
   return contentp;
  }


  public void paint(Graphics g)
  {
   layoutchildren();
   try
     {
      int width = getWidth()-framewidth/2;
      int height = getHeight()-framewidth/2;

      int textwidth = g.getFontMetrics().charsWidth(title.toCharArray(), 0, title.length());
      int textheight = g.getFontMetrics().getMaxAscent();

      g.setColor(getBackground());
      g.fillRect(0,0,getWidth(), getHeight());

        // draw translucent outher frame
      java.awt.Point p[] = new java.awt.Point[]
/*         {
          new java.awt.Point(frameedgesize, framewidth/2),
          new java.awt.Point(width-frameedgesize, framewidth/2),
          new java.awt.Point(width, frameedgesize),
          new java.awt.Point(width, height-frameedgesize),
          new java.awt.Point(width-frameedgesize, height),
          new java.awt.Point(frameedgesize, height),
          new java.awt.Point(framewidth/2, height-frameedgesize),
          new java.awt.Point(framewidth/2, frameedgesize),
          new java.awt.Point(frameedgesize, framewidth/2)
         };*/
         {
          new java.awt.Point(outherframeedgesize+outherframeoffset, framewidth/2+outherframeoffset),
          new java.awt.Point(width-outherframeedgesize-outherframeoffset, framewidth/2+outherframeoffset),
          new java.awt.Point(width-outherframeoffset, outherframeedgesize+outherframeoffset),
          new java.awt.Point(width-outherframeoffset, height-outherframeedgesize-outherframeoffset),
          new java.awt.Point(width-outherframeedgesize-outherframeoffset, height-outherframeoffset),
          new java.awt.Point(outherframeedgesize+outherframeoffset, height-outherframeoffset),
          new java.awt.Point(framewidth/2+outherframeoffset, height-outherframeedgesize-outherframeoffset),
          new java.awt.Point(framewidth/2+outherframeoffset, outherframeedgesize+outherframeoffset),
          new java.awt.Point(outherframeedgesize+outherframeoffset, framewidth/2+outherframeoffset)
         };
      GraphicHelper.drawSmoothPolyline(p, framewidth, GraphicHelper.getSelectedOutherBorderColor(), g);

         // draw the 2 border-frames
      p = new java.awt.Point[]
         {
          new java.awt.Point(width/2              , frameedgesize-framewidth/2),
          new java.awt.Point(width/2+frameedgesize, 1+framewidth/2),
          new java.awt.Point(width-1-frameedgesize, 1+framewidth/2),
          new java.awt.Point(width-1              , 1+frameedgesize+framewidth/2),
          new java.awt.Point(width-1              , height/2),
          new java.awt.Point(width-frameedgesize+framewidth/2  , frameedgesize+height/2),
          new java.awt.Point(width-frameedgesize+framewidth/2  , frameedgesize-framewidth/2),
          new java.awt.Point(width/2              , frameedgesize-framewidth/2)
         };
      GraphicHelper.fillSmoothPolyline(p, framewidth, GraphicHelper.getSelectedInnerBorderColor(), GraphicHelper.getSelectedFillColor(), g);

      p = new java.awt.Point[]
         {
          new java.awt.Point(frameedgesize-framewidth/2   , height/2),
          new java.awt.Point(1+framewidth/2               , height/2+frameedgesize),
          new java.awt.Point(1+framewidth/2               , height-1-frameedgesize),
          new java.awt.Point(1+frameedgesize+framewidth/2 , height-1),
          new java.awt.Point(width/2                      , height-1),
          new java.awt.Point(frameedgesize+width/2        , height-frameedgesize+framewidth/2 ),
          new java.awt.Point(frameedgesize-framewidth/2   , height-frameedgesize+framewidth/2),
          new java.awt.Point(frameedgesize-framewidth/2   , height/2)
         };
      GraphicHelper.fillSmoothPolyline(p, framewidth, GraphicHelper.getSelectedInnerBorderColor(), GraphicHelper.getSelectedFillColor(), g);


          // draw Title
      g.setColor(GraphicHelper.getSelectedInnerBorderColor());

      g.drawString(title, (width/2)-textwidth+(width/4), (textheight/2)+frameedgesize/2);


      paintChildren(g);
     }
   catch(Exception e)
     {
      e.printStackTrace();
     }
  };

 //--------- externaline and internalize to/from XML------------


 private class MyChainedDocumentHandler extends ChainedDocumentHandler
 {
  public void endElement(String p0) throws SAXException
  {
   super.endElement(p0);
  }

  public void startElement(String p0, AttributeList p1) throws SAXException
  {
   if(getLevel()==0)
     {
      if(!p0.equalsIgnoreCase("ARWindow"))
        throw new SAXException("expected: <ARWindow>, found: <"+p0+">");

     }
   else
   if(getLevel()==1)  // ignore deeper levels, that may exist dues to serialization of content that we do not know of yet
     {
        // ignore other children, that may be added in later versions of this format
      if(p0.equalsIgnoreCase("child"))
        {
         String classname = p1.getValue("classname");
         String title     = p1.getValue("title");
         try
           {
            Component c = (Component)Class.forName(classname).newInstance();
            if(c instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)c).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }
            addContent(c, title);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add content of type \""+classname+"\", "+e.getMessage());
           }
        }
     }
   super.startElement(p0, p1);
  }
 }

 public void externalizeToXML(java.io.PrintWriter pw)
 {
  pw.print("<ARWindow>");

  if(getContent() != null)
    {
     Component c = getContent();
     //TODO: don't externalize representations of services
/*
     pw.print("<child classname=\"");
        pw.print(getContent().getClass().toString());
     pw.print("\" title=\"");
     pw.print(title);
     pw.println("\">");

     if(c instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)c).externalizeToXML(pw);
       }
       //TODO: how to do this for AWT/Swing -components?
     pw.print("</child>");*/
    };

  pw.println("</ARWindow>");
 }

 public ChainedDocumentHandler internalizeFromXML()
 {
  this.removeAll();
  this.contentp = null;
  this.title = "";
  return new MyChainedDocumentHandler();
 }
}