/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-20, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-03-01, Marcus Wolschon<Suran@gmx.net>)
  *       - added serialization/deserialization
  * 0.0.3 (2002-03-02, Marcus Wolschon<Suran@gmx.net>)
  *       - added getFreeARWindow  and getAnyARWindow
  * 0.0.4 (2002-03-03, Marcus Wolschon<Suran@gmx.net>)
  *       - fixed XML-externalize
  *       - tested internalizing
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.UI.ARUI.widgets;

   // JDK
import java.awt.*;
import javax.swing.*;
  // project
import jaugment2.helpers.Logger;
import jaugment2.UI.ARUI.xml.*;
  // SAX
import org.xml.sax.*;

/**
 * Title:        ARUI
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class Block extends JPanel implements ARUIWidgetIF
{
 private static final int framewidth = 2;
 private static final boolean debug = true;

  public Block()
  {
   setBackground(new java.awt.Color( GraphicHelper.getFillColor().getRed(), GraphicHelper.getFillColor().getGreen(), GraphicHelper.getFillColor().getBlue()) );
   setForeground(GraphicHelper.getInnerBorderColor());

   setOpaque(false);
  }


  public void paint(java.awt.Graphics g)
  {
//   System.err.println("IconSlot paint bound="+getBounds());
   try
     {
      int width = getWidth()-framewidth/2;
      int height = getHeight()-framewidth/2;
//      g.setColor(getBackground());
//      g.fillRect(0,0,getWidth(), getHeight());
      java.awt.Point p[] = new java.awt.Point[]
         {
          new java.awt.Point(framewidth/2, framewidth/2),
          new java.awt.Point(width, framewidth/2),
          new java.awt.Point(width, framewidth/2),
          new java.awt.Point(width, height),
          new java.awt.Point(framewidth/2, height),
          new java.awt.Point(framewidth/2, framewidth/2),
          new java.awt.Point(framewidth/2+5, framewidth/2)
         };
      GraphicHelper.fillSmoothPolyline(p, framewidth, getForeground(), GraphicHelper.getFillColor(), g);
      if(content != null)
        {
         content.setBounds(framewidth/2, framewidth/2, width-framewidth/2, height-framewidth/2);
         paintChildren(g);

         if(debug)
         if(!content.isDisplayable())
           Logger.warning.println("WARNING[Block]: paint() content is not displayable parent="+content.getParent());
        }
     }
   catch(Exception e)
     {
      e.printStackTrace();
     }
  };



  private Component content;
  public void addContent(Component c)
  {
//   if(debug)Logger.debug.println("DEBUG[Block]: addContent(["+c.getClass().getName()+"]) getForeground()="+getForeground().toString());
   if(content != null)
     remove(content);
   this.content = c;
   add(c, null);
   c.setBackground(new java.awt.Color( GraphicHelper.getFillColor().getRed(), GraphicHelper.getFillColor().getGreen(), GraphicHelper.getFillColor().getBlue()) );
   c.setForeground(getForeground());
  }

  public Component getContent()
  {
   return this.content;
  }



 // ---------------------- ARUIWidgetIF ------------------------
 public IconSlot getFreeIconSlot()
 {
  return null;
 }
 public Block getFreeBlock()
 {
  if(getComponentCount()<1)
     return this;
  return null;
 }



 public ARWindow getFreeARWindow()
 {
  return ARUIWidgetHelper.getFreeARWindow(this);
 }
 public ARWindow getAnyARWindow()
 {
  return ARUIWidgetHelper.getFreeARWindow(this);
 }


 //--------- externaline and internalize to/from XML------------


 private class MyChainedDocumentHandler extends ChainedDocumentHandler
 {
  public void endElement(String p0) throws SAXException
  {
   super.endElement(p0);
  }

  public void startElement(String p0, AttributeList p1) throws SAXException
  {
   if(getLevel()==0)
     {
      if(!p0.equalsIgnoreCase("Block"))
        throw new SAXException("expected: <Block>, found: <"+p0+">");

     }
   else
   if(getLevel()==1)  // ignore deeper levels, that may exist dues to serialization of content that we do not know of yet
     {
        // ignore other children, that may be added in later versions of this format
      if(p0.equalsIgnoreCase("child"))
        {
         String classname = p1.getValue("classname");
         try
           {
            Component c = (Component)Class.forName(classname).newInstance();
            if(c instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)c).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }
            addContent(c);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add content of type \""+classname+"\", "+e.getMessage());
           }
        }
     }
   super.startElement(p0, p1);
  }
 }

 public void externalizeToXML(java.io.PrintWriter pw)
 {
  pw.print("<Block>");

  if(getContent() != null)
    {
     Component c = getContent();

     //TODO: don't externalize representations of services
/*     pw.print("<child classname=\"");
        pw.print(getContent().getClass().getName());
     pw.print("\">");

     if(c instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)c).externalizeToXML(pw);
       }
       //TODO: how to do this for AWT/Swing -components?
     pw.print("</child>");*/
    };

  pw.println("</Block>");
 }

 public ChainedDocumentHandler internalizeFromXML()
 {
  this.removeAll();
  this.content = null;
  return new MyChainedDocumentHandler();
 }
}
