/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-21, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-02-22, Marcus Wolschon<Suran@gmx.net>)
  *       - toggleState no longer causes the first icon to be drawn in all instances
  * -------------------------------------------------------------------
  * TODO:
  */
package jaugment2.UI.ARUI.widgets;

   // project
import jaugment2.helpers.Logger;

/**
 * Title:        ARUI
 * Description:  Widget, that is drawn for an icon of an application
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class IconWidget extends javax.swing.JPanel
{
 private static final boolean debug = true;

  //------------------------- events -------------------------


  public static interface IconWidgetListener
  {
   /**
    * returns true if the icon shall be placed into an ACTIVATED state
    */
   public boolean AppActivated(IconWidget widget, Object representation, net.jini.core.lookup.ServiceID serviceid, String title);


   public void AppDeActivated(IconWidget widget, Object representation, net.jini.core.lookup.ServiceID serviceid) throws java.beans.PropertyVetoException;
  };

  private java.util.LinkedList listeners = new java.util.LinkedList();

  public void addIconWidgetListener(IconWidgetListener l)
  {
   listeners.add(l);
  }

  public void removeIconWidgetListener(IconWidgetListener l)
  {
   listeners.remove(l);
  }


  //------------------------- state -------------------------

  public static final int STATE_DEACTIVATED = 0;
  public static final int STATE_ACTIVATED = 1;
  private int state = STATE_DEACTIVATED;
  public int getState()
  {
   return state;
  }
  public void toggleState()
  {
   if(debug)Logger.debug.println("DEBUG[IconWidget] toggleState() state="+state);
   if(state==STATE_DEACTIVATED)
     {
      state = STATE_ACTIVATED;
      java.util.Iterator iter = listeners.iterator();
      while(iter.hasNext())
        {
         ((IconWidgetListener)iter.next()).AppActivated(this, representation, serviceid, getToolTipText());
        }
//     repaint(10);
     }
   else
     {
      state = STATE_DEACTIVATED;
      java.util.Iterator iter = listeners.iterator();
      try
        {
         while(iter.hasNext())
           {
            ((IconWidgetListener)iter.next()).AppDeActivated(this, representation, serviceid);
           }
//     repaint(10);
        }
      catch(java.beans.PropertyVetoException e)
        {
         toggleState();
        }
     }
  }
  public void setState(int s)
  {
   if(state != s)
     toggleState();
  }


  //------------------------- UI -------------------------

  private java.awt.event.MouseAdapter mouselistener = new java.awt.event.MouseAdapter()
  {
   public void mouseReleased(java.awt.event.MouseEvent evt)
    {
     try
       {
        toggleState();
       }
     catch(Exception e)
       {
        e.printStackTrace(Logger.err);
       }
    }
  };



// private java.awt.Image icon;
  private javax.swing.ImageIcon icon = null;
  private String addName;
  private Object representation;
  private net.jini.core.lookup.ServiceID serviceid;

  public IconWidget(java.awt.Image icon, String appname, Object representation, net.jini.core.lookup.ServiceID serviceid)
  {
   setToolTipText(appname);
   this.representation = representation;
   this.serviceid = serviceid;
   this.addMouseListener(mouselistener);
   if(icon!=null)
     this.icon = new javax.swing.ImageIcon(icon);

   setOpaque(true);
  }

  public IconWidget(javax.swing.ImageIcon icon, String appname, Object representation, net.jini.core.lookup.ServiceID serviceid)
  {
   setToolTipText(appname);
   this.representation = representation;
   this.serviceid = serviceid;
   this.addMouseListener(mouselistener);
   this.icon = icon;

   setOpaque(true);
  }




  //------------------------- Graphics -------------------------

  public void paint(java.awt.Graphics g)
  {
   try
     {
//      Logger.debug.println("DEBUG[IconWidget]: paint() bounds="+getBounds()+" getWidth()="+getWidth());


      if(icon == null || icon.getImageLoadStatus() != java.awt.MediaTracker.COMPLETE)
        {
//         Logger.debug.println("DEBUG[IconWidget]: paint() painting yellow, we have no icon");

         g.setColor(new java.awt.Color( GraphicHelper.getFillColor().getRed(), GraphicHelper.getFillColor().getGreen(), GraphicHelper.getFillColor().getBlue()) );
         g.fillRect(getX(), getY(), getWidth(),getHeight());

         g.setColor(IconSlot.bordercolor);
         g.drawString("?", getX()-g.getFontMetrics().charWidth('?')+getWidth()/2, getHeight());
         return;
        }
      g.drawImage(icon.getImage(), getX(), getY(), getX()+getWidth(), getY()+getHeight(), 0, 0, icon.getIconWidth(), icon.getIconHeight(),  icon.getImageObserver());
     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  };
}