/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: Augmented-Reality UI
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-03-01, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-03-02, Marcus Wolschon<Suran@gmx.net>)
  *       - added getFreeARWindow  and getAnyARWindow
  * 0.0.3 (2002-03-03, Marcus Wolschon<Suran@gmx.net>)
  *       - fixed XML-externalizing
  *       - tested internalizing
  * -------------------------------------------------------------------
  * TODO:
  */

package jaugment2.UI.ARUI.widgets;

   // Swing
import java.awt.*;
import javax.swing.*;
   // project
import jaugment2.helpers.Logger;
import jaugment2.UI.ARUI.xml.*;
   // SAX
import org.xml.sax.*;

/**
 * Title:        ARUI
 * Description:  widget, that alows to have just child-widgets with a layout
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */
public class Panel extends JPanel implements ARUIWidgetIF
{

  public Panel()
  {
  }





 public IconSlot getFreeIconSlot()
 {
  return ARUIWidgetHelper.getFreeIconSlot(this);
 }
 public Block getFreeBlock()
 {
  return ARUIWidgetHelper.getFreeBlock(this);
 }

 public ARWindow getFreeARWindow()
 {
  return ARUIWidgetHelper.getFreeARWindow(this);
 }
 public ARWindow getAnyARWindow()
 {
  return ARUIWidgetHelper.getFreeARWindow(this);
 }




 //--------- keep book about used layout-constrains ------------

 private java.util.HashMap layoutContrains = new  java.util.HashMap();


 public void add(Component comp, Object constraints)
 {
  layoutContrains.put(comp, constraints);
  super.add(comp, constraints);
 }


 public void add(Component comp, Object constraints, int index)
  {
  layoutContrains.put(comp, constraints);
  super.add(comp, constraints, index);
 }


 //--------- externaline and internalize to/from XML------------


 private class MyChainedDocumentHandler extends ChainedDocumentHandler
 {
  public void endElement(String p0) throws SAXException
  {
   super.endElement(p0);
  }

  public void startElement(String p0, AttributeList p1) throws SAXException
  {
   if(getLevel()==0)
     {
      if(!p0.equalsIgnoreCase("Panel"))
        throw new SAXException("expected: <Panel>, found: <"+p0+">");

      String layout = p1.getValue("layoutmanager");
      if(layout ==null || layout.equals("null"))
        {
         setLayout(null);
        }
      else
      try
        {
         setLayout((LayoutManager)Class.forName(layout).newInstance());
        }
      catch(Exception e)
        {
         throw new SAXException("illegal value for attribute layoutmanager \""+layout+"\", "+e.getMessage());
        }

     }
   else
   if(getLevel()==1)  // ignore deeper levels, that may exist dues to serialization of content that we do not know of yet
     {
        // ignore other children, that may be added in later versions of this format
      if(p0.equalsIgnoreCase("child"))
        {
         String classname = p1.getValue("classname");
         String constrain = p1.getValue("constrain");

         if(classname==null)
            throw new SAXException("attribute classname missing for <child>");

         try
           {
            Component c = (Component)Class.forName(classname).newInstance();
            if(c==null)
               throw new SAXException("cannot add child of type \""+classname+"\" because such a class does not exist in my classpath");

            if(c instanceof ARUIWidgetIF)
              {
               ChainedDocumentHandler childhandler =((ARUIWidgetIF)c).internalizeFromXML();
               childhandler.setParser(getParser(), this);
               getParser().setDocumentHandler(childhandler);
              }
              //TODO: internalize AWT/Swing -components
            add(c, constrain);
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
            throw new SAXException("cannot add child of type \""+classname+"\", "+e.getMessage());
           }
        }
     }
   super.startElement(p0, p1);
  }
 }

 public void externalizeToXML(java.io.PrintWriter pw)
 {
  pw.print("<Panel layoutmanager=\"");
  pw.print(getLayout()==null?null:getLayout().getClass().getName());
  pw.println("\">");

  for(int i=0; i<getComponentCount(); i++)
    {
     Component c = getComponent(i);

     pw.print("<child constrain=\"");
        pw.print(layoutContrains.get(c).toString());
     pw.print("\" classname=\"");
        pw.print(c.getClass().getName());
     pw.println("\">");

     if(c instanceof ARUIWidgetIF)
       {
        ((ARUIWidgetIF)c).externalizeToXML(pw);
       }
       //TODO: how to do this for AWT/Swing -components?
     pw.println("</child>");
    };

  pw.println("</Panel>");
 }

 public ChainedDocumentHandler internalizeFromXML()
 {
  this.setLayout(null);
  this.removeAll();
  layoutContrains = new  java.util.HashMap();
  return new MyChainedDocumentHandler();
 }
}