/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: UIManagerBean
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2002-02-08, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * 0.0.2 (2002-02-09, Marcus Wolschon<Suran@gmx.net>)
  *       - implemented some of it's fnctionality
  * 0.0.3 (2002-02-20, Marcus Wolschon<Suran@gmx.net>)
  *       - untested support for icons
  * 0.0.4 (2002-02-24, Marcus Wolschon<Suran@gmx.net>)
  *       - checks package-versions for UIDescriptors
  * -------------------------------------------------------------------
  * TODO:
  *       - implement everything
  */
package jaugment2.UI.UIManager;

  //JINI
import net.jini.core.lookup.ServiceTemplate;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceID;
import net.jini.core.lease.Lease;
import net.jini.core.entry.Entry;
import net.jini.lease.LeaseRenewalManager;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.UnknownEventException;
  //serviceui-draft-standart
import net.jini.lookup.ui.factory.JComponentFactory;
import net.jini.lookup.ui.factory.PanelFactory;
import net.jini.lookup.ui.attribute.UIFactoryTypes;
import net.jini.lookup.ui.attribute.RequiredPackages;
import net.jini.lookup.entry.UIDescriptor;
import net.jini.lookup.entry.Name;
import net.jini.lookup.entry.ServiceType;
import net.jini.admin.Administrable;
   // Project
import jaugment2.helpers.Logger;
import jaugment2.UI.swingUI.SwingUIRepresentable;
import jaugment2.UI.textUI.TextUIRepresentable;
import jaugment2.UI.swingUI.TextUIApaptorPanel;
import jaugment2.UI.textUI.TextUI.ProcessAdaptor;

/**
 * <p>Title: RepresentationSeeker</p>
 * <p>Description: Helper-class doing the looking for the best representation(s) for the UIs</p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */
public class RepresentationSeeker implements UIManagerEventListener
{
  private static final boolean debugMessages = true;


  private boolean multipleRepresentations;
  public void setMultipleRepresentations(boolean multipleRepresentations)
  {
    this.multipleRepresentations = multipleRepresentations;
  }

  public boolean getMultipleRepresentations()
  {
    return multipleRepresentations;
  }

  public boolean hasMultipleRepresentations()
  {
    return multipleRepresentations;
  }



  private java.util.Vector baseClasses = new java.util.Vector();
  private java.util.Vector factories = new java.util.Vector();

  private static class FactoryDescriptor
  {
   private java.lang.Class baseclass;
   private java.lang.reflect.Method factoryMethod;
   private Object factoryMethodArgs[];
   public FactoryDescriptor(java.lang.Class baseclass, java.lang.reflect.Method factoryMethod, Object factoryMethodArgs[])
   {
    this.baseclass = baseclass;
    this.factoryMethod = factoryMethod;
    this.factoryMethodArgs = factoryMethodArgs;
   }

   public boolean matches(java.lang.Class c)
   {
    if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker.FactoryDescriptor] matches() baseclass="+baseclass.getName()+" c="+c.getName());
    return(baseclass.isAssignableFrom(c));
   }

   public Object invoke(Object o, ServiceItem ServiceItem) throws Exception
   {
    Object args[] = new Object[factoryMethodArgs.length];

    System.arraycopy(factoryMethodArgs, 0, args, 0, factoryMethodArgs.length);

    for(int i=0; i<args.length; i++)
     if(args[i] == null)
        args[i] = ServiceItem;
    try
      {
       return factoryMethod.invoke(o, args);
      }
    catch(Exception e)
      {
       Logger.err.println("ERROR[RepresentationSeeker.FactoryDescriptor] invoke() o is of class ["+o.getClass().getName()+"] factoryMethod's class is ["+factoryMethod.getDeclaringClass().getName()+"]");
       e.printStackTrace(Logger.err);
       return null;
      }
   }

  }

  /**
   *
   * @param baseclass base-class or interface the representation must implement
   */
  public void addRepresentationType(java.lang.Class baseclass) //TODO: priority
  {
   baseClasses.add(baseclass);
  }

  /**
   *
   * @param baseclass base-class or interface the Factory for the representation must implement
   * @param factoryMethodArgs arguments. null is replaced by the ServiceItem
   */
  public void addRepresentationFactoryType(java.lang.Class baseclass, java.lang.reflect.Method factoryMethod, Object factoryMethodArgs[])
  {
   factories.add( new FactoryDescriptor(baseclass, factoryMethod, factoryMethodArgs) );
  }

  public RepresentationSeeker()
  {
  }




  public java.awt.Component getAdministrativePanel(Object UI)
  {
   return null;
  }

  /**
   * Informs RepresentationSeeker that no event shall be thrown for this AdministrativePanel,
   * if the corresponding service vanishes.
   * @param c
   */
  public void doneWithAdministrativePanel( java.awt.Component c)
  {
  }


  //======================== UIManagerEventListener =========================

  private ServiceID createRandomServiceID()
  {
   return new ServiceID((long)(Long.MAX_VALUE*Math.random()), (long)(Long.MAX_VALUE*Math.random()));
  }


   public void localProcessStarted(UIManagerEventEvent e)
   {
    if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker:listener] localProcessStarted");

    newApp(new ServiceItem(createRandomServiceID(), e.proc, new Entry[] {}), true);
   }

   public void ApplicationFound(UIManagerEventEvent e)
   {
    if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker:listener] ApplicationFound");
    if(e.item != null)
      {
       if(e.item.serviceID == null)
         e.item.serviceID = createRandomServiceID();
        newApp(e.item, false);
      }
   }

   public void ApplicationVanished(UIManagerEventEvent e)
   {
    if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker:listener] ApplicationVanished");
    if(e.item != null)
      {
       if(e.item instanceof ServiceItem)
         {
          lostApp((ServiceItem)e.item);
         }
      }
   }

  //========================   Listeners    =========================

  private void fireRepresentationFound(ServiceDescriptor service)
  {
   services.put(service.getClass(), service);

   if(listener != null)
   try
     {
      listener.RepresentationsFound(service.getRepresentations(), service.getIcon(), service.getName(), service.getID());
     }
   catch(Exception e)
     {
      e.printStackTrace();
     }
  }

  private void fireRepresentationLost(ServiceDescriptor service)
  {
   if(listener == null)
     return;

   Object o[] = service.getRepresentations();
   if(o != null)
   for(int i=0; i<o.length; i++)
      listener.RepresentationLost(o[i], service.getName(), service.getID());

  }

  private RepresentationSeekerEventListener listener;
  public void setListener(RepresentationSeekerEventListener l)
  {this.listener = l;}

  //======================== implementation =========================

  private java.util.HashMap services = new java.util.HashMap();

  private static class ServiceDescriptor
  {
   private ServiceID id;
   public ServiceDescriptor(ServiceID id)
   {
    this.id = id;
   }

   public ServiceID getID()
   {return id;}

   private String name = null;

   public String getName()
   {return name;}

   public void setName(String n)
   {this.name = n;}


   private java.util.LinkedList representations = new java.util.LinkedList();

   public int getRepresentationsCount()
   {
    return representations.size();
   }

   public void addRepresentation(Object o)
   {
    representations.add(o);
   }

   public Object[] getRepresentations()
   {
    return representations.toArray();
   }

   private java.awt.Image icon = null;
   public java.awt.Image getIcon()
   {
    return icon;
   }
   public void setIcon(java.awt.Image icon)
   {
    this.icon = icon;
   }

  }



  public static boolean isDockin(java.awt.Component c)
  {
//   if(debugMessages)jaugment2.helpers.Logger.debug.println("DEBUG: [RepresentationSeeker] isDockin() test1="+(c instanceof javax.swing.JComponent)+" test2="+((javax.swing.JComponent)c).getMaximumSize().getHeight()+"<40");
   return (c instanceof javax.swing.JComponent && ((javax.swing.JComponent)c).getMaximumSize().getHeight()<40);
  }


  /**
   * helper-function to find an icon for a service
   */
  private void findIcon(ServiceItem app, ServiceDescriptor service)
  {
   try
     {
      //TODO: find icon atached as Attribute first

      if(app.attributeSets != null)
      for(int i=0; i<app.attributeSets.length; i++)
        if(app.attributeSets[i] instanceof ServiceType)
          {
           ServiceType st = (ServiceType)app.attributeSets[i];
           java.awt.Image icon = st.getIcon(java.beans.BeanInfo.ICON_COLOR_32x32); //TODO: types
           if(icon != null)
             {
              service.setIcon(icon);
              return;
             }
          }



          // try to find an icon in the Beandescriptor for the Proxy that was sent to us

      if(debugMessages)jaugment2.helpers.Logger.debug.println("DEBUG: [RepresentationSeeker] findIcon() app.service.getClass()=["+app.service.getClass().getName()+"]");
      java.beans.BeanInfo bi = java.beans.Introspector.getBeanInfo(app.service.getClass());
      if(bi == null || (
                        bi.getIcon(java.beans.BeanInfo.ICON_COLOR_16x16)==null
                               &&
                        bi.getIcon(java.beans.BeanInfo.ICON_COLOR_32x32)==null
                               &&
                        bi.getIcon(java.beans.BeanInfo.ICON_MONO_16x16)==null
                               &&
                        bi.getIcon(java.beans.BeanInfo.ICON_MONO_32x32)==null
                       ))
      try
        {
          // no luck, maybe we can guess about the service-class and have it in the classpath
         String classname = app.service.getClass().getName();
         if(classname.endsWith("Proxy"))
           {
            if(classname.indexOf("Service") == -1)
              {
               classname = classname.substring(0, classname.length()-5)+"Service";
              }
            else
              classname = classname.substring(0, classname.length()-5);
           }
         else
           return;
         if(debugMessages)jaugment2.helpers.Logger.debug.println("DEBUG: [RepresentationSeeker] findIcon() app.service.getClass(){second try}=["+classname+"]");
         bi = java.beans.Introspector.getBeanInfo(Class.forName(classname));
        }
      catch(Exception e)
        {
         if(debugMessages)e.printStackTrace(Logger.debug);
         return;
        }
      catch(IllegalAccessError x)
        {
         if(debugMessages)x.printStackTrace(Logger.debug);
         return;
        }

      if(bi == null)
        return;


       java.awt.Image icon =   bi.getIcon(java.beans.BeanInfo.ICON_COLOR_16x16);
       if(icon != null)
         {
          service.setIcon(icon);
          return;
         }
                      icon =   bi.getIcon(java.beans.BeanInfo.ICON_MONO_16x16);
       if(icon != null)
         {
          service.setIcon(icon);
          return;
         }
                      icon =   bi.getIcon(java.beans.BeanInfo.ICON_COLOR_32x32);
       if(icon != null)
         {
//TODO: shrink
          service.setIcon(icon);
          return;
         }
                      icon =   bi.getIcon(java.beans.BeanInfo.ICON_MONO_32x32);
       if(icon != null)
         {
//TODO: shrink
          service.setIcon(icon);
          return;
         }

     }
   catch(Exception e)
     {
      e.printStackTrace(Logger.err);
     }
  }


  /**
   *
   * @param app - app.service must not be null, app.serviceID must be unique
   * @param bringToTop
   * @return
   */
  private boolean newApp(ServiceItem app, boolean bringToTop)
  {
   if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker] newApp");
   try
     {
      ServiceDescriptor service  = new ServiceDescriptor(app.serviceID);

      findIcon(app, service);

      if(app.attributeSets != null)
      for(int i=0; i<app.attributeSets.length; i++)
         {
          Entry o = app.attributeSets[i];

          if(o instanceof Name)
            {
             Name n = (Name)o;
             service.setName(n.name);
             if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker] newApp - name is "+n.name);
             continue;
            }

          if(baseClasses.contains(o.getClass()))
            {
             service.addRepresentation(o);
             if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker] newApp - baseclass "+o.getClass()+" is in the list");
             if(!multipleRepresentations)
               {
                // fire and forget
                fireRepresentationFound(service);
                return true;
               }
             continue;
            }

          if(o instanceof UIDescriptor)
            {
             UIDescriptor uid = (UIDescriptor)o;
             if(!checkUIDescriptor(uid))
               continue;
             Object factory = uid.factory.get();
             Class factoryclass = factory.getClass();  //TODO: how to do this w/o de-serialization

             if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker] newApp - Attribute is a UIDescriptor of type "+factoryclass.getName());

             boolean found = false;
             for(int f=0; f<factories.size(); f++)
                {
                 FactoryDescriptor desc = (FactoryDescriptor)factories.get(f);
                 try
                   {
                    if(!desc.matches(factoryclass))
                      continue;
                    if(debugMessages)Logger.debug.println("DEBUG: [RepresentationSeeker] newApp - a FactoryDescriptor matches");
                    Object o2 = desc.invoke(factory, app);
                    if(o2 != null)
                      {
                       service.addRepresentation(o2);
                       if(!multipleRepresentations)
                         {
                          // fire and forget
                          fireRepresentationFound(service);
                          return true;
                         }
                       else
                         {
                          found=true;
                          break;
                         }
                      }
                   }
                 catch(Exception e)
                   {
                    e.printStackTrace(Logger.err);
                   }
                }
             if(found)
               continue;

             if(factories.contains(o.getClass()))
               {
                service.addRepresentation(o);
                continue;
               }

            }
         }

      // try a local process

      if(app.service instanceof Process)
        {
         if(baseClasses.contains(SwingUIRepresentable.class))
           {
            service.addRepresentation( new TextUIApaptorPanel(new ProcessAdaptor((Process)app.service)) );
           }
         else
         if(baseClasses.contains(TextUIRepresentable.class))
           {
            service.addRepresentation( new ProcessAdaptor((Process)app.service) );
           }
        }

       if(service.getRepresentationsCount()==0)
         return false;
      fireRepresentationFound(service);
      return true;
     }
   catch(Exception e)
     {
      e.printStackTrace();
     }
   return false;
  }

  private void lostApp(ServiceItem app)
  {
   try
     {
      ServiceDescriptor service = (ServiceDescriptor)services.get(app.serviceID);
      if(service != null)
         fireRepresentationLost(service);
     }
   catch(Exception e)
     {
      e.printStackTrace();
     }
  }




  /**
   * checks the attributes of the UIDescriptor.
   * If we are unable to use this UI it returns false.
   */
  private boolean checkUIDescriptor(UIDescriptor uid)
  {
   java.util.Iterator iter = uid.attributes.iterator();
   while(iter.hasNext())
     {
      Object o = iter.next();
      if(o instanceof RequiredPackages)
        {
         RequiredPackages req = (RequiredPackages)o;
         java.util.Iterator pkgiter = req.getRequiredPackages().keySet().iterator();
         while(iter.hasNext())
           {
            try
              {
               String pkg = iter.next().toString();
               Package p = Package.getPackage(pkg);
               if(req.getVersion(pkg)!=null)
               if(req.getVersion(pkg).equals("0")) // any version
               if(!p.isCompatibleWith(req.getVersion(pkg)))
                 return false;
              }
            catch(Exception e)
              {
               e.printStackTrace(Logger.err);
               return false;
              }
           }
        }
     }
   return true;
  }

}
